package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
	"gorm.io/gorm"
)

type PendaftaranRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewPendaftaranRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *PendaftaranRepo {
	return &PendaftaranRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *PendaftaranRepo) NewPendaftaran(pendaftaran models.PendaftaranTA) (output models.PendaftaranTA, err error) {
	if pendaftaran.IdTopik == "" {
		topic := models.Topik{
			ID:        uuid.New().String(),
			Judul:     pendaftaran.Topik.Judul,
			Deskripsi: pendaftaran.Topik.Deskripsi,
			PengajuID: pendaftaran.IdDosen,
		}

		pendaftaran.IdTopik = topic.ID
		pendaftaran.Topik = topic
	}

	pendaftaran.ID = uuid.New().String()
	pendaftaran.WaktuPengiriman = time.Now()
	pendaftaran.Status = models.Pendaftaran_ta_status_enum("NOT_ASSIGNED")
	result := repo.DBWrite.Create(&pendaftaran)
	if result.Error != nil {
		return models.PendaftaranTA{}, result.Error
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) GetPendaftaran() (output []models.PendaftaranTA, err error) {
	var pendaftaran []models.PendaftaranTA
	result := repo.DBRead.Find(&pendaftaran)
	if result.Error != nil {
		return nil, err
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) DeletePendaftaran(id string) (models.PendaftaranTA, error) {
	batch := models.PendaftaranTA{ID: id}
	err := repo.DBWrite.Delete(&batch).Error

	if err != nil {
		return batch, err
	}

	return batch, err
}

func (repo *PendaftaranRepo) GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error) {
	var pendaftaran []models.PendaftaranTA
	result := repo.DBRead.Where("id_dosen = ?", id).Find(&pendaftaran)
	if result.Error != nil {
		return nil, err
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error) {
	var pendaftaran []models.PendaftaranTA
	var dosen models.Pengguna
	var data []entity.PendaftaranGetResponse
	var topik models.Topik

	res := repo.DBRead.Where("id_mahasiswa = ?", id).Find(&pendaftaran)
	if res.Error != nil {
		return data, res.Error
	}

	for _, pen := range pendaftaran {
		res = repo.DBRead.Where("id = ?", pen.IdDosen).Find(&dosen).Limit(1)
		if res.Error != nil {
			return data, res.Error
		}

		res = repo.DBRead.Where("id = ?", pen.IdTopik).Find(&topik).Limit(1)
		if res.Error != nil {
			return data, res.Error
		}

		var dataTemp = entity.PendaftaranGetResponse{
			Id:              pen.ID,
			Status:          string(pen.Status),
			InterviewAt:     pen.InterviewAt,
			DecidedAt:       pen.DecidedAt,
			WaktuPengiriman: pen.WaktuPengiriman,
			Judul:           topik.Judul,
			Deskripsi:       topik.Deskripsi,
			IdDosen:         pen.IdDosen,
			NamaDosen:       dosen.Nama,
			EmailDosen:      dosen.Email,
		}

		data = append(data, dataTemp)
	}

	if res.Error != nil {
		return data, err
	}
	return data, nil
}

func (repo *PendaftaranRepo) SetStatus(id string, val string) (output []models.PendaftaranTA, err error) {
	var pendaftaran models.PendaftaranTA
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.Status = models.Pendaftaran_ta_status_enum(val)
	if val == "APPROVED" || val == "REJECTED" {
		now := time.Now()
		pendaftaran.DecidedAt = now
	}

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []models.PendaftaranTA{pendaftaran}, nil
}

func (repo *PendaftaranRepo) SetTimeInterview(id string, val time.Time) (output []models.PendaftaranTA, err error) {
	var pendaftaran models.PendaftaranTA
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.InterviewAt = val
	pendaftaran.Status = models.Pendaftaran_ta_status_enum("INTERVIEW")

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []models.PendaftaranTA{pendaftaran}, nil
}

func (repo *PendaftaranRepo) GetPendaftaranByPeriode(idPeriode string) (output []models.PendaftaranTA, err error) {
	var periode []models.PendaftaranTA
	result := repo.DBRead.Where("id_periode = ?", idPeriode).Find(&periode)
	if result.Error != nil {
		return nil, err
	}
	return periode, nil
}

func (repo *PendaftaranRepo) GetRekapPendaftaranDosen(idDosen string) ([]entity.RekapPendaftaran, error) {
	var res []entity.RekapPendaftaran
	err := repo.DBRead.Table("pendaftaran_ta").
		Select("pendaftaran_ta.id, pendaftaran_ta.id_mahasiswa, pendaftaran_ta.status, pendaftaran_ta.interview_at, pengguna.nama, pengguna.nim").
		Joins("JOIN pengguna on pendaftaran_ta.id_mahasiswa = pengguna.id").
		Where("pendaftaran_ta.id_dosen = ?", idDosen).
		Scan(&res).Error
	if err != nil {
		return nil, err
	}

	return res, nil
}

func (repo *PendaftaranRepo) GetDetailRekapPendafataran(idDosen string, idMahasiswa string) (entity.DetailRekapPendaftara, error) {
	var res entity.DetailRekapPendaftara
	err := repo.DBRead.Table("topik").
		Select("topik.judul, topik.deskripsi, pendaftaran_ta.status, pendaftaran_ta.interview_at, pendaftaran_ta.waktu_pengiriman, pendaftaran_ta.jalur_pilihan").
		Joins("JOIN pendaftaran_ta ON topik.id = pendaftaran_ta.id_topik").
		Where("pendaftaran_ta.id_dosen = ? AND pendaftaran_ta.id_mahasiswa = ?", idDosen, idMahasiswa).
		Scan(&res).Error
	if err != nil {
		return entity.DetailRekapPendaftara{}, err
	}

	return res, nil
}

func (repo *PendaftaranRepo) UpdateInterview(interviewAt string, idDosen string, idMahasiswa string) error {
	updates := map[string]interface{}{
		"interview_at": interviewAt,
		"status":       "INTERVIEW",
	}

	if err := repo.DBWrite.Table("pendaftaran_ta").
		Where("id_dosen = ? AND id_mahasiswa = ?", idDosen, idMahasiswa).
		Updates(updates).Error; err != nil {
		return err
	}

	return nil
}

func (repo *PendaftaranRepo) UpdateStatus(status string, idDosen string, idMahasiswa string) error {
	var pendaftaranID string
	updates := map[string]interface{}{
		"status": status,
	}

	if err := repo.DBWrite.Table("pendaftaran_ta").
		Where("id_dosen = ? AND id_mahasiswa = ?", idDosen, idMahasiswa).
		Updates(updates).Error; err != nil {
		return err
	}

	if err := repo.DBWrite.Table("pendaftaran_ta").
		Select("id").
		Where("id_dosen = ? AND id_mahasiswa = ?", idDosen, idMahasiswa).
		Scan(&pendaftaranID).Error; err != nil {
		return err
	}

	dosenBimbingan := entity.DosenBimbingan{
		ID:            uuid.New().String(),
		PendaftaranID: pendaftaranID,
		DosenID:       idDosen,
	}

	if err := repo.DBWrite.Table("dosen_bimbingan").Create(&dosenBimbingan).Error; err != nil {
		return err
	}

	return nil
}

func (repo *PendaftaranRepo) UpdateStatusById(status string, idPendaftaran string) error {
	updates := map[string]interface{}{
		"status": status,
	}

	if err := repo.DBWrite.Table("pendaftaran_ta").
		Where("id = ?", idPendaftaran).
		Updates(updates).Error; err != nil {
		return err
	}

	return nil
}

func (repo *PendaftaranRepo) UpdateDosbing(idDosbing string, idPendaftaran string) error {
	pendaftaranTaUpdates := map[string]interface{}{
		"id_dosen": idDosbing,
	}
	dosenBimbinganUpdate := map[string]interface{}{
		"dosen_id": idDosbing,
	}

	if err := repo.DBWrite.Table("pendaftaran_ta").
		Where("id = ?", idPendaftaran).
		Updates(pendaftaranTaUpdates).Error; err != nil {
		return err
	}

	if err := repo.DBWrite.Table("dosen_bimbingan").
		Where("pendaftaran_id = ?", idPendaftaran).
		Updates(dosenBimbinganUpdate).Error; err != nil {
		return err
	}

	return nil
}

func (repo *PendaftaranRepo) CountApproved(idDosen string) (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("status = ? AND id_dosen = ?", "APPROVED", idDosen).
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}
func (repo *PendaftaranRepo) CountProses(idDosen string) (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("id_dosen = ? AND status = ? ", idDosen, "NOT_ASSIGNED").
		Or("id_dosen = ? AND status = ? ", idDosen, "INTERVIEW").
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}

func (repo *PendaftaranRepo) CountRejected(idDosen string) (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("status = ?  AND id_dosen = ?", "REJECTED", idDosen).
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}

func (repo *PendaftaranRepo) GetListofDosbing() ([]entity.Pengguna, error) {
	var dosbings []entity.Pengguna

	result := repo.DBRead.Table("pengguna").
		Joins("JOIN LATERAL unnest(roles) AS role ON true").
		Where("role = ?", models.S1_PEMBIMBING).
		Group("pengguna.id").
		Find(&dosbings)
	if result.Error != nil {
		return nil, result.Error
	}

	return dosbings, nil
}

func (repo *PendaftaranRepo) CountApprovedAll() (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("status = ?", "APPROVED").
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}
func (repo *PendaftaranRepo) CountProsesAll() (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("status = ? ", "NOT_ASSIGNED").
		Or("status = ? ", "INTERVIEW").
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}

func (repo *PendaftaranRepo) CountRejectedAll() (int64, error) {
	var count int64

	if err := repo.DBRead.Table("pendaftaran_ta").
		Where("status = ?", "REJECTED").
		Count(&count).Error; err != nil {
		return 0, err
	}

	return count, nil
}

func (repo *PendaftaranRepo) GetDosbingName(idMahasiswa string) (string, error) {
	var dosbingName string

	query := `
		SELECT p.nama 
		FROM pengguna p 
		JOIN dosen_bimbingan db ON p.id = db.dosen_id 
		JOIN pendaftaran_ta pt ON db.pendaftaran_id = pt.id 
		WHERE pt.id_mahasiswa = ? 
		ORDER BY pt.waktu_pengiriman DESC 
		LIMIT 1;
	`

	if err := repo.DBRead.Raw(query, idMahasiswa).Scan(&dosbingName).Error; err != nil {
		return "", err
	}

	return dosbingName, nil
}

func (repo *PendaftaranRepo) GetAllPendaftaran() ([]entity.RekapPendaftaran, error) {
	var res []entity.RekapPendaftaran
	err := repo.DBRead.Table("pendaftaran_ta").
		Select("pendaftaran_ta.id, pendaftaran_ta.id_mahasiswa, pendaftaran_ta.status, pendaftaran_ta.interview_at, pengguna.nama, pengguna.nim").
		Joins("JOIN pengguna on pendaftaran_ta.id_mahasiswa = pengguna.id").
		Scan(&res).Error
	if err != nil {
		return nil, err
	}

	return res, nil
}

func (repo *PendaftaranRepo) GetRiwayatPendaftaran(idMahasiswa string) ([]entity.RiwayatPendaftaran, error) {
	var res []entity.RiwayatPendaftaran
	err := repo.DBRead.Table("pendaftaran_ta").
		Select("pendaftaran_ta.id, pendaftaran_ta.jalur_pilihan, pendaftaran_ta.status, pendaftaran_ta.interview_at,  pendaftaran_ta.waktu_pengiriman, topik.judul, topik.deskripsi").
		Joins("JOIN topik on pendaftaran_ta.id_topik = topik.id").
		Where("pendaftaran_ta.id_mahasiswa = ?", idMahasiswa).
		Scan(&res).Error
	if err != nil {
		return nil, err
	}

	return res, nil
}

func (repo *PendaftaranRepo) GetDosbingForPendaftaran(idMahasiswa string) ([]entity.DosenPembimbing, error) {
	var res []entity.DosenPembimbing
	err := repo.DBRead.Table("pendaftaran_ta").
		Select("pendaftaran_ta.id as id_pendaftaran, pengguna.id, pengguna.nama").
		Joins("JOIN pengguna on pendaftaran_ta.id_dosen = pengguna.id").
		Where("pendaftaran_ta.id_mahasiswa = ?", idMahasiswa).
		Scan(&res).Error
	if err != nil {
		return nil, err
	}

	return res, nil
}

func (repo *PendaftaranRepo) GetAllDosbing() ([]entity.DosenPembimbings, error) {
	var res []entity.DosenPembimbings
	err := repo.DBRead.Table("pengguna").
		Select("pengguna.id, pengguna.nama, pengguna.email").
		Where("? = ANY(pengguna.roles)", "S1_PEMBIMBING").
		Scan(&res).Error
	if err != nil {
		return nil, err
	}

	return res, nil
}
