package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TIMTAPendaftaranSidSemHandler struct {
	pendaftaransidsemUsecase usecase.PendaftaranSidSemUsecase
}

func NewTIMTAPendaftaranSidSemHandler(cfg config.PendaftaranSidSemTransportConfig) *TIMTAPendaftaranSidSemHandler {
	pendaftaransidsemRepository := repository.NewPendaftaranSidSemRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaransidsemUsecase := usecase.NewPendaftaranSidSemUseCase(pendaftaransidsemRepository)

	return &TIMTAPendaftaranSidSemHandler{
		pendaftaransidsemUsecase: pendaftaransidsemUsecase,
	}

}

func (t *TIMTAPendaftaranSidSemHandler) MountTIMTA(group *echo.Group) {
	group.GET("/pendaftaran-sidsem", t.GetPendaftaran)
	group.GET("/detail-sidsem", t.GetSidSemDetail)
	group.PATCH("/pendaftaran-ditolak", t.EditDitolak)
	group.PATCH("/update-ruangan-sidsem", t.UpdateRuanganSidSem)
	group.PATCH("/update-jadwal-sidsem", t.UpdateJadwalSidsem)
	group.PATCH("/update-dosuji-sidsem", t.UpdateDosujiSidsem)
	group.GET("/get-dosuji", t.GetDosuji)

}

func (t *TIMTAPendaftaranSidSemHandler) GetPendaftaran(c echo.Context) error {
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		size = 10
	}
	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		offset = 0
	}
	jadwal, err := t.pendaftaransidsemUsecase.GetPendaftaran(size, offset)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", jadwal))
}

func (t *TIMTAPendaftaranSidSemHandler) EditDitolak(c echo.Context) error {
	request := new(entity.EditDitolakRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditDitolak(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}

func (t *TIMTAPendaftaranSidSemHandler) GetSidSemDetail(c echo.Context) error {
	idSidsem := c.QueryParam("id")
	jadwal, err := t.pendaftaransidsemUsecase.GetSidSemDetailTA(idSidsem)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Sidsem Detail successfully retrieved", jadwal))
}

func (t *TIMTAPendaftaranSidSemHandler) UpdateRuanganSidSem(c echo.Context) error {
	request := new(entity.UpdateRuanganReq)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	err := t.pendaftaransidsemUsecase.UpdateRuanganSidSem(request.NamaRuangan, request.IdSidSem)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Ruangan successfully updated", nil))
}

func (t *TIMTAPendaftaranSidSemHandler) UpdateJadwalSidsem(c echo.Context) error {
	request := new(entity.UpdateJadwalReq)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	err := t.pendaftaransidsemUsecase.UpdateJadwalSidsem(request.WaktuMulai, request.IdSidSem)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal successfully updated", nil))
}

func (t *TIMTAPendaftaranSidSemHandler) GetDosuji(c echo.Context) error {
	idSidsem := c.QueryParam("id")
	dosUjis, err := t.pendaftaransidsemUsecase.GetDosuji(idSidsem)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Dosujis successfully retrieved", dosUjis))
}

func (t *TIMTAPendaftaranSidSemHandler) UpdateDosujiSidsem(c echo.Context) error {
	request := new(entity.UpdateDosenReq)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	err := t.pendaftaransidsemUsecase.UpdateDosujiSidsem(request.Dosujis, request.IdSidSem)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Dosujis successfully updated", err))
}
