package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminRuanganHandler struct {
	ruanganUsecase usecase.RuanganUsecase
}

func NewAdminRuanganHandler(cfg config.RuanganTransportConfig) *AdminRuanganHandler {
	ruanganRepository := repository.NewRuanganRepository(cfg.DBWrite, cfg.DBRead)
	ruanganUsecase := usecase.NewRuanganUseCase(ruanganRepository)

	return &AdminRuanganHandler{
		ruanganUsecase: ruanganUsecase,
	}

}

func (t *AdminRuanganHandler) MountAdmin(group *echo.Group) {
	group.GET("/ruangan", t.GetRuangan)
	group.GET("/ruangan-id", t.GetRuanganById)
	group.GET("/ruangan-all", t.GetAllRuangan)
	group.POST("/ruangan", t.AddRuangan)
}

// AddRuangan adds a new entity for Ruangan
// @Summary Add new Ruangan entity
// @Description Adds a new entity for ruangan
// @Tags Ruang
// @Accept json
// @Produce json
// @Param batch body entity.Ruangan true "Ruangan object"
// @Success 200 {object} entity.Ruangan "Ruangan successfully added"
// @Router /api/admin/ruangan [post]
func (t *AdminRuanganHandler) AddRuangan(c echo.Context) error {
	request := new(entity.Ruangan)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	ruangan, err := t.ruanganUsecase.AddRuangan(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Ruangan successfully added", ruangan))
}

// GetRuangan retrieves a list of ruangan in table ruangan
// @Summary Retrieve list of ruangan
// @Description Retrieves a list of ruangan in table ruangan with specific offset and limit
// @Tags Ruang
// @Accept json
// @Produce json
// @Param limit query int false "Limit number of ruangan per page"
// @Param offset query int false "Offset for pagination"
// @Success 200 {object} entity.Ruangan "Ruangan successfully added"
// @Router /api/admin/ruangan [get]
func (t *AdminRuanganHandler) GetRuangan(c echo.Context) error {
	param, err := utils.GetLimitOffset(c.QueryParams())
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	ruangan, err := t.ruanganUsecase.GetRuangan(param)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Ruangan successfully retreived", ruangan))
}

// GetRuangan retrieves a ruangan entity with specific ID
// @Summary Retrieve list of ruangan
// @Description Retrieves a ruangan entity in table ruangan with specific ID
// @Tags Ruang
// @Accept json
// @Produce json
// @Param id query string true "ID of Ruangan"
// @Success 200 {object} entity.Ruangan "Ruangan successfully added"
// @Router /api/admin/ruangan-id [get]
func (t *AdminRuanganHandler) GetRuanganById(c echo.Context) error {
	id := c.QueryParam("id")

	ruangan, err := t.ruanganUsecase.GetRuanganById(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Ruangan successfully retreived", ruangan))
}

// GetRuangan retrieves a list of ruangan in table ruangan
// @Summary Retrieve list of ruangan
// @Description Retrieves a list of ruangan in table ruangan
// @Tags Ruang
// @Accept json
// @Produce json
// @Success 200 {object} entity.Ruangan "Ruangan successfully added"
// @Router /api/admin/ruangan-all [get]
func (t *AdminRuanganHandler) GetAllRuangan(c echo.Context) error {
	ruangan, err := t.ruanganUsecase.GetAllRuangan()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Ruangan successfully retreived", ruangan))
}
